from tkinter import *
from time import sleep
import math

class constants(object):
    width = 600
    height = 500
    sizeDia = 18
    sizeho = 15
    size = "600x700"

class obstacle(object):
    def __init__(self, x0,y0,x1,y1,kind):
        self.x0 = x0
        self.x1 = x1
        self.y0 = y0
        self.y1 = y1
        self.kind = kind

    def make(self, canvas):
        if self.kind == "C":
            canvas.create_oval(self.x0,self.y0,self.x1,self.y1, fill='black')
        elif self.kind == "R":
            canvas.create_rectangle(self.x0,self.y0,self.x1,self.y1, fill='black')

    def check(self, px, py):
        if self.kind == 'C':
            cx = (self.x0 + self.x1) / 2
            cy = (self.y0 + self.y1) / 2
            d = (self.x0 - self.x1) / 2
            d = math.fabs(self.x0 - cx)
            if (px-cx)**2 + (py-cy)**2  < (constants.sizeDia + d)**2:
                return False
            return True
        elif self.kind == 'R':
            if (self.x0 <= px+constants.sizeho and
            px-constants.sizeho < self.x1 and
            self.y0 <= py+constants.sizeho and
            py-constants.sizeho <= self.y1):
                return False
            return True
        return False
            

class Application(Frame):
    def __init__(self, master = None):
        Frame.__init__(self, master)
        self.grid()
        self.createObstacles()
        self.createWidgets()

    def createObstacles(self):
        self.obstacles = [ obstacle(0,200, 400,250,"R"),
                           obstacle(450,200, 600,250,"R"),
                           obstacle(400,450, 300, 350, "C"),
                           obstacle(210,100,250, 200,"R"),
                           obstacle(100,50, 140,90,"C"),
                           obstacle(200,10,240,50,"C"),
                           obstacle(200,400,250,450,"C"),
                           obstacle(100,300,150,350,"C"),
                           obstacle(380,450,420,500,"R"),
                           obstacle(400,100,450,150,"R"),
                           obstacle(400,300,450,350,"R"),
                           obstacle(500,350,600,400,"R"),
                           obstacle(0,100,50,100,"R")]

    def createWidgets(self):
        self.quitButton = Button(self, text='Exit', command=self.quit)
        self.quitButton.grid(column=4, row=0)
        self.goButton = Button(self, text='Execute', command=self.go)
        self.goButton.grid(column=0, row=0)
        self.label = Text(self, height=1, width=5 )
        self.label.grid(column=1, row=0)
        self.label.insert(END,"PC")
        self.lab1 = Label(self, height=1, text="Wait")
        self.lab1.grid(column=2, row=0)
        self.spinbox = Spinbox(self, values = (0.1, 0.2, 0.3, 0.4, 0.5, 0.75, 1.0, 1.5, 2.0))
        self.spinbox.grid(column=3, row=0)
        self.canvas = Canvas(self, height=constants.height, width=constants.width, bg='#FFFFFF')
        self.canvas.grid(column=0, row=1, columnspan=5, sticky=(N, W, E, S))
        for obs in self.obstacles:
            obs.make(self.canvas)
        self.goal = self.canvas.create_rectangle(constants.width - 50, constants.height - 50,
                                                 constants.width, constants.height, fill = "green")
        self.N = PhotoImage(file='./N.gif')
        self.NE = PhotoImage(file='./NE.gif')
        self.E = PhotoImage(file='./E.gif')
        self.SE = PhotoImage(file='./SE.gif')
        self.S = PhotoImage(file='./S.gif')
        self.SW = PhotoImage(file='./SW.gif')
        self.W = PhotoImage(file='./W.gif')
        self.NW = PhotoImage(file='./NW.gif')
        self.star = PhotoImage(file='./star.gif')
        self.finish = PhotoImage(file='./finish.gif')
        self.x = 20
        self.y = 20
        self.obj = self.canvas.create_image(self.x, self.y, image=self.E, anchor=CENTER)
        self.state = 2
        self.pc = 0
 

    def go(self):
        self.label.delete(1.0,END)
        self.label.insert(END,"0")
        self.file = open("program.txt", "r")
        self.x = 20
        self.y = 20
        self.state = 2
        self.pc = 0
        self.go = True
        while self.go:
            self.label.delete(1.0,END)
            self.label.insert(END,str(self.pc))
            self.pc += 1
            self.draw()
            if self.x > constants.width - 50 and self.y > constants.height - 50:
                self.canvas.create_image(constants.width/2, constants.height/2, image=self.finish,
                                         anchor= CENTER)
                self.go = False
            if self.check() == False:
                self.canvas.delete(self.obj)
                if self.x < 100:
                    self.x = 100
                if self.x > 600 - 100:
                    self.x = 500
                if self.y > 800 - 100:
                    self.y = 700
                if self.y < 100:
                    self.y = 100
                self.obj = self.canvas.create_image(self.x, self.y, image=self.star, anchor=CENTER)
                self.canvas.update()
                break
            sleep(float(self.spinbox.get()))
        self.file.close()

    def draw(self):
        line = self.file.readline()
        if len(line) == 0:
            self.go = False
            return
        if line.startswith("R"):
            self.state = (self.state+1) % 8
            print("R", self.state)
        elif line.startswith("L"):
            self.state = (self.state+7) % 8
            print("L", self.state)
        elif line.startswith("M"):
            if self.state == 0:
                self.y -= 10
            elif self.state == 1:
                self.x += int(10.0/math.sqrt(2))
                self.y -= int(10.0/math.sqrt(2))
            elif self.state == 2:
                self.x += 10
            elif self.state == 3:
                self.x += int(10.0/math.sqrt(2))
                self.y += int(10.0/math.sqrt(2))
            elif self.state == 4:
                self.y += 10
            elif self.state == 5:
                self.x -= int(10.0/math.sqrt(2))
                self.y += int(10.0/math.sqrt(2))
            elif self.state == 6:
                self.x -= 10
            elif self.state == 7:
                self.x -= int(10.0/math.sqrt(2))
                self.y -= int(10.0/math.sqrt(2))
            print("M", self.x, self.y)
        self.canvas.delete(self.obj)
        if self.state==0:
            self.obj = self.canvas.create_image(self.x, self.y, image=self.N, anchor=CENTER)
        elif self.state==1:
            self.obj = self.canvas.create_image(self.x, self.y, image=self.NE, anchor=CENTER)
        elif self.state==2:
            self.obj = self.canvas.create_image(self.x, self.y, image=self.E, anchor=CENTER)
        elif self.state==3:
            self.obj = self.canvas.create_image(self.x, self.y, image=self.SE, anchor=CENTER)
        elif self.state==4:
            self.obj = self.canvas.create_image(self.x, self.y, image=self.S, anchor=CENTER)
        elif self.state==5:
            self.obj = self.canvas.create_image(self.x, self.y, image=self.SW, anchor=CENTER)
        elif self.state==6:
            self.obj = self.canvas.create_image(self.x, self.y, image=self.W, anchor=CENTER)
        elif self.state==7:
            self.obj = self.canvas.create_image(self.x, self.y, image=self.NW, anchor=CENTER)
        self.canvas.update()
        return

    def check(self):
        for obs in self.obstacles:
            if obs.check(self.x, self.y) == False:
                return False
        if (self.x < constants.sizeho or
        self.x > constants.width - constants.sizeho
        or self.y < constants.sizeho or
        self.y > constants.height - constants.sizeho):
            return False
        return True           
                            
    def quit(self):
        self.master.destroy()

app = Application()
app.master.title("Walking Robot")
app.master.geometry(constants.size)
app.mainloop()







    

        
